@@ -23,8 +23,7 @@ from .event import (LeftHolelessEvent as LeftEvent,
                     RightShapedEvent as RightEvent,
                     events_to_connectivity)
 from .events_queue import HolelessEventsQueue as EventsQueue
-from .hints import (Region,
-                    SegmentEndpoints)
+from .hints import Region
 from .operands import HolelessOperand
 from .sweep_line import BinarySweepLine as SweepLine
 from .unpacking import (unpack_mix,
@@ -34,7 +33,6 @@ from .unpacking import (unpack_mix,
 from .utils import (all_equal,
                     contour_to_oriented_edges_endpoints,
                     endpoints_to_segments,
-                    pairwise,
                     shrink_collinear_vertices,
                     to_endpoints,
                     to_regions_x_max)
@@ -74,10 +72,12 @@ class Operation(ABC):
                                             if (event.from_first
                                                 is below_event.from_first)
                                             else below_event.interior_to_left)
-        event.in_result = self.in_result(event)
+        event.from_shaped_result = self.from_shaped_result(event)
 
     def events_to_regions(self, events: Iterable[Event]) -> Sequence[Region]:
-        events = sorted([event for event in events if event.primary.in_result],
+        events = sorted([event
+                         for event in events
+                         if event.primary.from_shaped_result],
                         key=self._events_queue.key)
         for index, event in enumerate(events):
             event.position = index
@@ -121,8 +121,8 @@ class Operation(ABC):
                     False)
 
     @abstractmethod
-    def in_result(self, event: LeftEvent) -> bool:
-        """Detects if event will be presented in result of the operation."""
+    def from_shaped_result(self, event: LeftEvent) -> bool:
+        """Detects if event is a part of resulting shaped geometry."""
 
     def process_event(self,
                       event: Event,
@@ -177,33 +177,28 @@ class CompleteIntersection(Operation):
         events = sorted(self.sweep(),
                         key=self._events_queue.key)
         points = []  # type: List[Point]
-        endpoints = []  # type: List[SegmentEndpoints]
         for start, same_start_events in groupby(events,
                                                 key=attrgetter('start')):
             same_start_events = list(same_start_events)
-            if not (any(event.is_left and event.in_result
+            if not (any(event.primary.wholly_in_complete_intersection
                         for event in same_start_events)
                     or all_equal(event.from_first
                                  for event in same_start_events)):
-                no_segment_found = True
-                for event, next_event in pairwise(same_start_events):
-                    if (event.from_first is not next_event.from_first
-                            and event.start == next_event.start
-                            and event.end == next_event.end):
-                        no_segment_found = False
-                        if event.is_left:
-                            endpoints.append(to_endpoints(next_event))
-                if no_segment_found and all(not event.primary.in_result
-                                            for event in same_start_events):
-                    points.append(start)
-        segments = endpoints_to_segments(endpoints, context)
+                points.append(start)
+        segments = endpoints_to_segments(
+                [to_endpoints(event)
+                 for event in events
+                 if (event.is_left
+                     and event.from_first
+                     and event.is_common_polyline_component)],
+                context)
         regions = self.events_to_regions(events)
         return unpack_mix(unpack_points(points, context),
                           unpack_segments(segments, context),
                           unpack_regions(regions, context),
                           context)
 
-    def in_result(self, event: LeftEvent) -> bool:
+    def from_shaped_result(self, event: LeftEvent) -> bool:
         return (event.inside
                 or not event.from_first and event.is_common_region_boundary)
 
@@ -253,7 +248,7 @@ class Intersection(Operation):
             self.process_event(event, result, sweep_line)
         return result
 
-    def in_result(self, event: LeftEvent) -> bool:
+    def from_shaped_result(self, event: LeftEvent) -> bool:
         return (event.inside
                 or not event.from_first and event.is_common_region_boundary)
 
