@@ -21,10 +21,12 @@ from ground.hints import (Contour,
 from reprit.base import generate_repr
 
 from . import bounding
-from .event import (LeftHoleyEvent as LeftEvent,
+from .event import (UNDEFINED_INDEX,
+                    LeftHoleyEvent as LeftEvent,
                     RightShapedEvent as RightEvent,
                     events_to_connectivity)
 from .events_queue import HoleyEventsQueue as EventsQueue
+from .hints import Orienteer
 from .operands import HoleyOperand
 from .sweep_line import BinarySweepLine as SweepLine
 from .unpacking import (unpack_mix,
@@ -81,7 +83,12 @@ class Operation(ABC):
                 else below_event)
         event.from_shaped_result = self.from_shaped_result(event)
 
-    def events_to_polygons(self, events: Iterable[Event]) -> Sequence[Polygon]:
+    def events_to_polygons(self, events: Sequence[Event]) -> Sequence[Polygon]:
+        if not events:
+            return []
+        max_start_index = events[-1].start_index
+        assert max_start_index != UNDEFINED_INDEX
+        assert all(event.start_index <= max_start_index for event in events)
         events = [event
                   for event in events
                   if event.primary.from_shaped_result]
@@ -94,15 +101,17 @@ class Operation(ABC):
         contour_cls, orienteer = context.contour_cls, context.angle_orientation
         contours = []  # type: List[Contour]
         connectivity = events_to_connectivity(events)
+        visited_positions = [UNDEFINED_INDEX] * (max_start_index + 1)
         for index, event in enumerate(events):
             if processed[index]:
                 continue
             contour_id = len(contours)
             _compute_relations(event, contour_id, are_internal, depths, holes,
                                parents)
-            vertices = _events_to_contour_vertices(event, events, contour_id,
-                                                   connectivity, processed)
-            shrink_collinear_vertices(vertices, orienteer)
+            contour_events = _to_contour_events(event, events, connectivity,
+                                                processed, visited_positions)
+            _process_contour_events(contour_events, contour_id, processed)
+            vertices = _contour_events_to_vertices(contour_events, orienteer)
             if depths[contour_id] % 2:
                 # holes will be in clockwise order
                 vertices[:] = vertices[:1] + vertices[:0:-1]
@@ -116,7 +125,8 @@ class Operation(ABC):
                         polygon_cls(contours[hole_index],
                                     [contours[hole_hole_index]
                                      for hole_hole_index in holes[hole_index]])
-                        for hole_index in holes[index])
+                        for hole_index in holes[index]
+                )
             else:
                 result.append(polygon_cls(contour,
                                           [contours[hole_index]
@@ -128,11 +138,13 @@ class Operation(ABC):
         for polygon in self.first.polygons:
             events_queue.register(
                     polygon_to_oriented_edges_endpoints(polygon, self.context),
-                    True)
+                    True
+            )
         for polygon in self.second.polygons:
             events_queue.register(
                     polygon_to_oriented_edges_endpoints(polygon, self.context),
-                    False)
+                    False
+            )
 
     @abstractmethod
     def from_shaped_result(self, event: LeftEvent) -> bool:
@@ -170,50 +182,32 @@ class Operation(ABC):
                 self.compute_fields(below_event, below_below_event)
                 self.compute_fields(event, below_event)
 
-    def sweep(self) -> Iterable[Event]:
+    def sweep(self) -> List[Event]:
         self.fill_queue()
         result = []
         sweep_line = SweepLine(self.context)
         events_queue = self._events_queue
-        while events_queue:
-            self.process_event(events_queue.pop(), result, sweep_line)
-        return result
-
-
-class Difference(Operation):
-    __slots__ = ()
-
-    def compute(self) -> Union_[Empty, Multipolygon, Polygon]:
-        context = self.context
-        first_box = context.polygons_box(self.first.polygons)
-        if bounding.disjoint_with(first_box,
-                                  context.polygons_box(self.second.polygons)):
-            return self.first.value
-        self.second.polygons = bounding.to_coupled_polygons(
-                first_box, self.second.polygons, context)
-        return (unpack_polygons(self.events_to_polygons(self.sweep()), context)
-                if self.second.polygons
-                else self.first.value)
-
-    def from_shaped_result(self, event: LeftEvent) -> bool:
-        return (event.outside
-                if event.from_first
-                else event.inside or event.is_common_polyline_component)
-
-    def sweep(self) -> Iterable[Event]:
-        self.fill_queue()
-        result = []
-        events_queue = self._events_queue
-        sweep_line = SweepLine(self.context)
-        first_x_max = to_polygons_x_max(self.first.polygons)
+        event = events_queue.pop()
+        start = event.start
+        start_index = event.start_index = 0
+        self.process_event(event, result, sweep_line)
         while events_queue:
             event = events_queue.pop()
-            if first_x_max < event.start.x:
-                break
+            if event.start != start:
+                start = event.start
+                start_index += 1
+            event.start_index = start_index
             self.process_event(event, result, sweep_line)
         return result
 
 
+def _contour_events_to_vertices(events: Sequence[Event],
+                                orienteer: Orienteer) -> List[Point]:
+    result = [events[0].start] + [event.end for event in events[:-1]]
+    shrink_collinear_vertices(result, orienteer)
+    return result
+
+
 class CompleteIntersection(Operation):
     __slots__ = ()
 
@@ -229,7 +223,8 @@ class CompleteIntersection(Operation):
         if not self.first.polygons:
             return context.empty
         self.second.polygons = bounding.to_intersecting_polygons(
-                first_box, self.second.polygons, context)
+                first_box, self.second.polygons, context
+        )
         if not self.second.polygons:
             return context.empty
         events = sorted(self.sweep(),
@@ -249,28 +244,79 @@ class CompleteIntersection(Operation):
                  if (event.is_left
                      and event.from_first
                      and event.is_common_polyline_component)],
-                context)
+                context
+        )
         polygons = self.events_to_polygons(events)
         return unpack_mix(unpack_points(points, context),
                           unpack_segments(segments, context),
-                          unpack_polygons(polygons, context),
-                          context)
+                          unpack_polygons(polygons, context), context)
 
     def from_shaped_result(self, event: LeftEvent) -> bool:
         return (event.inside
                 or not event.from_first and event.is_common_region_boundary)
 
-    def sweep(self) -> Iterable[Event]:
+    def sweep(self) -> List[Event]:
         self.fill_queue()
         result = []
         events_queue = self._events_queue
         sweep_line = SweepLine(self.context)
         min_max_x = min(to_polygons_x_max(self.first.polygons),
                         to_polygons_x_max(self.second.polygons))
+        event = events_queue.pop()
+        start = event.start
+        start_index = event.start_index = 0
+        self.process_event(event, result, sweep_line)
         while events_queue:
             event = events_queue.pop()
             if min_max_x < event.start.x:
                 break
+            if event.start != start:
+                start = event.start
+                start_index += 1
+            event.start_index = start_index
+            self.process_event(event, result, sweep_line)
+        return result
+
+
+class Difference(Operation):
+    __slots__ = ()
+
+    def compute(self) -> Union_[Empty, Multipolygon, Polygon]:
+        context = self.context
+        first_box = context.polygons_box(self.first.polygons)
+        if bounding.disjoint_with(first_box,
+                                  context.polygons_box(self.second.polygons)):
+            return self.first.value
+        self.second.polygons = bounding.to_coupled_polygons(
+                first_box, self.second.polygons, context
+        )
+        return (unpack_polygons(self.events_to_polygons(self.sweep()), context)
+                if self.second.polygons
+                else self.first.value)
+
+    def from_shaped_result(self, event: LeftEvent) -> bool:
+        return (event.outside
+                if event.from_first
+                else event.inside or event.is_common_polyline_component)
+
+    def sweep(self) -> List[Event]:
+        self.fill_queue()
+        result = []
+        events_queue = self._events_queue
+        sweep_line = SweepLine(self.context)
+        first_x_max = to_polygons_x_max(self.first.polygons)
+        event = events_queue.pop()
+        start = event.start
+        start_index = event.start_index = 0
+        self.process_event(event, result, sweep_line)
+        while events_queue:
+            event = events_queue.pop()
+            if first_x_max < event.start.x:
+                break
+            if event.start != start:
+                start = event.start
+                start_index += 1
+            event.start_index = start_index
             self.process_event(event, result, sweep_line)
         return result
 
@@ -285,11 +331,13 @@ class Intersection(Operation):
         if bounding.disjoint_with(first_box, second_box):
             return context.empty
         self.first.polygons = bounding.to_coupled_polygons(
-                second_box, self.first.polygons, context)
+                second_box, self.first.polygons, context
+        )
         if not self.first.polygons:
             return context.empty
         self.second.polygons = bounding.to_coupled_polygons(
-                first_box, self.second.polygons, context)
+                first_box, self.second.polygons, context
+        )
         return (unpack_polygons(self.events_to_polygons(self.sweep()), context)
                 if self.second.polygons
                 else context.empty)
@@ -298,17 +346,25 @@ class Intersection(Operation):
         return (event.inside
                 or not event.from_first and event.is_common_region_boundary)
 
-    def sweep(self) -> Iterable[Event]:
+    def sweep(self) -> List[Event]:
         self.fill_queue()
         result = []
         events_queue = self._events_queue
         sweep_line = SweepLine(self.context)
         min_max_x = min(to_polygons_x_max(self.first.polygons),
                         to_polygons_x_max(self.second.polygons))
+        event = events_queue.pop()
+        start = event.start
+        start_index = event.start_index = 0
+        self.process_event(event, result, sweep_line)
         while events_queue:
             event = events_queue.pop()
             if min_max_x < event.start.x:
                 break
+            if event.start != start:
+                start = event.start
+                start_index += 1
+            event.start_index = start_index
             self.process_event(event, result, sweep_line)
         return result
 
@@ -380,32 +436,43 @@ def _compute_relations(event: LeftEvent,
     are_internal.append(is_internal)
 
 
-def _events_to_contour_vertices(cursor: LeftEvent,
-                                events: Sequence[LeftEvent],
-                                contour_id: int,
-                                connectivity: Sequence[int],
-                                processed: List[bool]) -> List[Point]:
-    contour_start = cursor.start
-    result, contour_events = [contour_start], [cursor]
-    opposite_position = cursor.right.position
-    vertices_positions = {contour_start: 0}
+def _to_contour_events(event: LeftEvent,
+                       events: Sequence[LeftEvent],
+                       connectivity: Sequence[int],
+                       processed: Sequence[bool],
+                       visited_positions: List[int]) -> List[Event]:
+    result = [event]
+    visited_positions[event.start_index] = 0
+    opposite_position = event.right.position
+    contour_start = event.start
+    cursor = event
     while cursor.end != contour_start:
-        vertex = cursor.end
-        if vertex in vertices_positions:
-            # vertices loop found, i.e. contour has self-intersection
-            previous_vertex_position = vertices_positions[vertex]
-            del result[previous_vertex_position:]
-            del contour_events[previous_vertex_position:]
+        if visited_positions[cursor.end_index] == UNDEFINED_INDEX:
+            visited_positions[cursor.end_index] = len(result)
         else:
-            vertices_positions[vertex] = len(result)
-        result.append(vertex)
+            # vertices loop found, i.e. contour has self-intersection
+            previous_vertex_position = visited_positions[cursor.end_index]
+            assert previous_vertex_position != 0
+            for event in result[previous_vertex_position:]:
+                visited_positions[event.end_index] = UNDEFINED_INDEX
+            del result[previous_vertex_position:]
         position = _to_next_position(opposite_position, processed,
                                      connectivity)
         if position is None:
             break
         cursor = events[position]
-        contour_events.append(cursor)
         opposite_position = cursor.opposite.position
+        result.append(cursor)
+    visited_positions[result[0].start_index] = UNDEFINED_INDEX
+    for event in result:
+        visited_positions[event.end_index] = UNDEFINED_INDEX
+    assert all(position == UNDEFINED_INDEX for position in visited_positions)
+    return result
+
+
+def _process_contour_events(contour_events: Iterable[Event],
+                            contour_id: int,
+                            processed: List[bool]) -> None:
     for event in contour_events:
         processed[event.position] = processed[event.opposite.position] = True
         if event.is_left:
@@ -414,7 +481,6 @@ def _events_to_contour_vertices(cursor: LeftEvent,
         else:
             event.opposite.from_in_to_out = True
             event.opposite.contour_id = contour_id
-    return result
 
 
 def _to_next_position(position: int,
